package be.iminds.ilabt.jfed.lowlevel.api.test;

import be.iminds.ilabt.jfed.testing.base.ApiTestConfig;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.dropwizard.jackson.Jackson;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Objects;

@JsonInclude(JsonInclude.Include.NON_DEFAULT)
@JsonIgnoreProperties(value={"@context"})
public class GPULabTestConfig implements ApiTestConfig {
    private static final Logger LOG = LoggerFactory.getLogger(GPULabTestConfig.class);

    @Nonnull private final String gpulabURL;
    @Nonnull private final String version;
    @Nonnull private final Integer minGPUs;
    @Nonnull private final Integer minCPUs;
    @Nonnull private final Integer minSlaves;
    @Nonnull private final Integer minClusters;

    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();

    public GPULabTestConfig() {
        this.gpulabURL = "https://dev.gpulab.ilabt.imec.be/";
        this.version = "dev";
        this.minGPUs = 1;
        this.minCPUs = 1;
        this.minSlaves = 1;
        this.minClusters = 1;
    }

    @JsonCreator
    public GPULabTestConfig(@JsonProperty("gpulabURL") @Nullable String gpulabURL,
                            @JsonProperty("version") @Nullable String version,
                            @JsonProperty("minGPUs") @Nullable Integer minGPUs,
                            @JsonProperty("minCPUs") @Nullable Integer minCPUs,
                            @JsonProperty("minSlaves") @Nullable Integer minSlaves,
                            @JsonProperty("minClusters") @Nullable Integer minClusters) {
        this.gpulabURL = gpulabURL == null ? new GPULabTestConfig().getGpulabURL() : gpulabURL;
        this.version = version == null ? new GPULabTestConfig().getVersion() : version;
        this.minGPUs = minGPUs == null ? new GPULabTestConfig().getMinGPUs() : minGPUs;
        this.minCPUs = minCPUs == null ? new GPULabTestConfig().getMinCPUs() : minCPUs;
        this.minSlaves = minSlaves == null ? new GPULabTestConfig().getMinSlaves() : minSlaves;
        this.minClusters = minClusters == null ? new GPULabTestConfig().getMinClusters() : minClusters;
    }



//    @JsonCreator
//    public static GPULabTestConfig createFromJsonObject(@Nonnull Map<String,Object> props) {
//        GPULabTestConfig res = MAPPER.convertValue(props, GPULabTestConfig.class);
//        return res;
//    }

    @Override
    public boolean isValid() {
        return gpulabURL.startsWith("https://");
    }

    @Nonnull
    @JsonProperty
    public String getGpulabURL() {
        return gpulabURL;
    }

    @Nonnull
    @JsonProperty
    public String getVersion() {
        return version;
    }

    @Nonnull
    public Integer getMinGPUs() {
        return minGPUs;
    }

    @Nonnull
    public Integer getMinCPUs() {
        return minCPUs;
    }

    @Nonnull
    public Integer getMinSlaves() {
        return minSlaves;
    }

    @Nonnull
    public Integer getMinClusters() {
        return minClusters;
    }

    @Nonnull
    @Override
    public String stringValue() {
        try {
            return MAPPER.writeValueAsString(this);
        } catch (JsonProcessingException e) {
            throw new RuntimeException("Failed to covert "+getClass().getName()+" to String", e);
        }
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        GPULabTestConfig that = (GPULabTestConfig) o;
        return gpulabURL.equals(that.gpulabURL) &&
                version.equals(that.version) &&
                minGPUs.equals(that.minGPUs) &&
                minCPUs.equals(that.minCPUs) &&
                minSlaves.equals(that.minSlaves) &&
                minClusters.equals(that.minClusters);
    }

    @Override
    public int hashCode() {
        return Objects.hash(gpulabURL, version, minGPUs, minCPUs, minSlaves, minClusters);
    }
}
